@@ -5,6 +5,7 @@ from urllib3.exceptions import ReadTimeoutError, SSLError as UrllibSSLError
 from urllib3.util.retry import Retry
 import warnings
 import gzip
+from base64 import decodestring
 
 # sentinal value for `verify_certs`.
 # This is used to detect if a user is passing in a value for `verify_certs`
@@ -72,6 +73,8 @@ class Urllib3HttpConnection(Connection):
         information.
     :arg headers: any custom http headers to be add to requests
     :arg http_compress: Use gzip compression
+    :arg cloud_id: The Cloud ID from ElasticCloud. Convient way to connect to cloud instances.
+        Other host connection params will be ignored.
     """
 
     def __init__(
@@ -92,9 +95,18 @@ class Urllib3HttpConnection(Connection):
         headers=None,
         ssl_context=None,
         http_compress=False,
+        cloud_id=None,
         **kwargs
     ):
 
+        if cloud_id:
+            cluster_name, cloud_id = cloud_id.split(":")
+            url, es_uuid, kibana_uuid = (
+                decodestring(cloud_id.encode("utf-8")).decode("utf-8").split("$")
+            )
+            host = "%s.%s" % (es_uuid, url)
+            port = "9243"
+            use_ssl = True
         super(Urllib3HttpConnection, self).__init__(
             host=host, port=port, use_ssl=use_ssl, **kwargs
         )
